@@ -78,7 +78,7 @@ static char *tzbuffer = NULL;
 
 /* prototypes */
 void usage(FILE *stream);
-void name_list(FILE *stream);
+void name_list(FILE *stream, const char what);
 void show_version(FILE *stream, const int show_long);
 void add_entry(struct shortid_list_s *tree, struct usage_flags *flag, const char *entry_name, const int vf);
 int has_entry(const struct shortid_list_s *tree);
@@ -132,6 +132,9 @@ usage(FILE *stream)
 	fputs("                 with TSCL, TSCS, TSCV for long, short, variable period status.\n", stream);
 	fputs("\n", stream);
 	fputs("-l             : list all the available status names and exit.\n", stream);
+	fputs("-la            : list only alarms.\n", stream);
+	fputs("-lw            : list only warnings.\n", stream);
+	fputs("-lf            : list only faults.\n", stream);
 	fputs("                 You may want to pipe the output to a pager.\n", stream);
 	fputs("\n", stream);
 	fputs("-z             : use local timezone insted of UTC.\n", stream);
@@ -167,25 +170,42 @@ show_version(FILE *stream, const int show_long)
 
 /* show list of available names */
 void
-name_list(FILE *stream)
+name_list(FILE *stream, const char what)
 {
 	struct melco_status_s *info;
 	int i, n;
+	int use;
 	n = melco_status_total();
 	fprintf(stream, "#name\t[type-shortid location id]\n");
 	for(i = 0; i < n; i++)
 		{
+			use = 1;
 			info = melco_status_ith(i);
-			if (info->mask)
+			switch(what)
 				{
-					int b, m;
-					m = info->mask;
-					for(b = 0; (m & 0x01) == 0; b++) m >>= 1;
-					fprintf(stream, "%s\t[%s-%04X %d:%d %07X]\n", info->name, info->type, info->shortid, info->offset, b, info->id);
+					case 'a':
+						if (!	info->isalarm) use = 0;
+						break;
+					case 'w':
+						if (!	info->iswarning) use = 0;
+						break;
+					case 'f':
+						if (!	info->isfault) use = 0;
+						break;
 				}
-			else
+			if (use)
 				{
-					fprintf(stream, "%s\t[%s-%04X %d %07X]\n", info->name, info->type, info->shortid, info->offset, info->id);
+					if (info->mask)
+						{
+							int b, m;
+							m = info->mask;
+							for(b = 0; (m & 0x01) == 0; b++) m >>= 1;
+							fprintf(stream, "%s\t[%s-%04X %d:%d %07X]\n", info->name, info->type, info->shortid, info->offset, b, info->id);
+						}
+					else
+						{
+							fprintf(stream, "%s\t[%s-%04X %d %07X]\n", info->name, info->type, info->shortid, info->offset, info->id);
+						}
 				}
 		}
 }
@@ -778,6 +798,7 @@ main(int argc, char *argv[])
 		int r;
 		char *entrybuf, *end_of_entry;
 		int entrybuflen, inputlen, more_in_line;
+		char what = 0;
 		entrybuflen = melco_status_word_len_max() + 2;	/* keyword + \n + \0 */
 		if ((entrybuf = (char *) malloc(entrybuflen)) == NULL)
 			{
@@ -785,7 +806,7 @@ main(int argc, char *argv[])
 				exit(EXIT_FAILURE);
 			}
 		dstdir = DEF_DSTDIR;
-		while((r = getopt(argc, argv, "Vhe:vqd:anlEzr:bt")) != -1)
+		while((r = getopt(argc, argv, "Vhe:vqd:anl::Ezr:bt")) != -1)
 			{
 				switch(r)
 					{
@@ -798,7 +819,22 @@ main(int argc, char *argv[])
 						free(entrybuf);
 						return EXIT_SUCCESS;
 					case 'l':	/* list */
-						name_list(stdout);
+						if (optarg)
+							{
+								switch(*optarg)
+									{
+										case 'a':
+										case 'w':
+										case 'f':
+											what = *optarg;
+											break;
+										default:
+											usage(stderr);
+											free(entrybuf);
+											return EXIT_FAILURE;
+									}
+								}
+						name_list(stdout, what);
 						free(entrybuf);
 						return EXIT_SUCCESS;
 					case 'E':	/* add entries from stdin */
