@@ -351,8 +351,10 @@ ASC15_TIME_to_t(const void *addr, const unsigned char mask __attribute__ ((unuse
 
 	memset(&tm, 0, sizeof(tm));
 	res = strptime(data->buf, "%Y%m%d%H%M%S", &tm);
-	if (!res) return NULL;	/* error in input time string */
+	if (!res || 14 != res - data->buf) return NULL;	/* error in input time string - should have read 14 bytes of 15 bytes except for the 0.1 sec digit*/
+
 	tv->tv_sec = timegm(&tm);
+	if (tv->tv_sec < MIN_MESSAGE_TIME || MAX_MESSAGE_TIME < tv->tv_sec) return NULL;	/* got a strange time - maybe due to parsing a sequence of numbers coinsidentally */
 
 	strncpy(secbuf, data->sep.sec+2, sizeof(data->sep.sec)-2);
 	secbuf[sizeof(data->sep.sec)-2] = '\0';
