@@ -12,6 +12,8 @@
 import sys
 import re
 import csv
+import os
+import string
 
 import mitsubishi
 
@@ -514,7 +516,19 @@ class TelStatusEntry(mitsubishi.TelStatusEntry):
 		tot = self.reader_function_name('t')
 		if not tot:
 			tot = 'NULL'
-		return '{"%s", "%s", 0x%s, 0x%s, %d, %d, %s, %s, %s, %s, %s, %s},\t/* %d */\n' % (name, ext, shortid, id, offset, bytes, mask, tod, toi, tos, tol, tot, number)
+		if self.isalarm:
+			isalarm = 1
+		else:
+			isalarm = 0
+		if self.iswarning:
+			iswarning = 1
+		else:
+			iswarning = 0
+		if self.isfault:
+			isfault = 1
+		else:
+			isfault = 0
+		return '{"%s", "%s", 0x%s, 0x%s, %d, %d, %s, %d, %d, %d, %s, %s, %s, %s, %s},\t/* %d */\n' % (name, ext, shortid, id, offset, bytes, mask, isalarm, iswarning, isfault, tod, toi, tos, tol, tot, number)
 
 	def hash_entry(self, number, entry_info_array_name):
 		return '"%s", &%s[%d]\n' % (self.keyword(), entry_info_array_name, number)
@@ -525,15 +539,208 @@ class TelStatusEntry(mitsubishi.TelStatusEntry):
 		r = "%s %-2.2s %s %05d %d" % (realshortid(t, s), extype(t, s), realshortid(t, s), self.byteoffset, self.bitoffset)
 		return r
 
-# we want to use TelStatusEntry defined in this file
-class TelStatusCsv(mitsubishi.TelStatusCsv):
+#class TelStatusCsv(mitsubishi.TelStatusCsv):
+	# parse
+#	def parse(self):
+#		r = []
+#		for col in csv.reader(file(self.path)):
+#			x = self.corrected(col)
+#			for c in self.corrected(col):
+#				e = TelStatusEntry(c[1], c[2], c[6], c[7], c[8], c[9], c[12], c[13], c[14], c[15], self.basename, {'ja':c[5]})
+#				r.append(e)
+#		return r
+
+# a CSV file from Mitsubishi
+class TelStatusCsv:
+	def __init__(self, path):
+		self.path = path
+		self.basename = os.path.basename(self.path).split('.')[0]
+
+	# Zernike polynominal terms
+	zernike_terms = [
+		'00', '11', '1m1', '20', '31', '3m1', '22', '2m2', '33', '3m3',
+		'40', '42', '4m2', '44', '4m4', '51', '5m1', '53', '5m3', '55',
+		'5m5', '60', '62', '6m2', '64', '6m4', '66', '6m6', '71', '7m1',
+		'73', '7m3', '75', '7m5', '77', '7m7', '80', '82', '8m2', '84',
+		'8m4', '86', '8m6', '88', '8m8', '91', '9m1', '93', '9m3', '95',
+		'9m5', '97', '9m7', '99', '9m9', '100', '102', '10m2', '104', '10m4',
+		'106', '10m6', '108', '10m8', '1010', '10m10', '', '', '', ''
+	]
+
+	# correction for typos in Mitsubishi CSV
+	# input:output
+	# col[0]	: entry number within the file
+	# col[1]	: `name' of the status (string)
+	# col[2]	: number of bits (int)
+	# col[3]	: positive/negative logic
+	# col[4]	: source interface
+	# col[5]	: description in Japanese (string, encoding depends upon input)
+	#	         Python >= 2.4 is needed to convert strings to UTF-8
+	#	         with a standard Python library.
+	# col[6]	: whether this is a source of alarm
+	# col[7]	: whether this is a source of warning
+	# col[8]	: whether this is a source of failure
+	# col[9]	: format of numeric data (upper-cased string)
+	# col[10] : exclusive signlals
+	# col[11] : remarks
+	# col[12] : status ID (zero-filled string)
+	# col[13] : E:event driven/variable, L:long period, S:short period (string)
+	# col[14] : byte offset (int)
+	# col[15] : bit offset or decimal position (string)
+	# returns a list of normalized and type-converted status entries
+
+	def corrected(self, col):
+		e = col[:]
+		# number of columns
+		if len(col) < 15:
+			return []
+		# correct O (Capital-O) to 0 (zero)
+		e[12] = re.sub('O', '0', e[12]) # MCP1MON.csv
+		e[14] = re.sub('O', '0', e[14]) # TLSCPCT2.csv and TLSCPENV.csv
+		e[15] = re.sub('O', '0', e[15]) # DOMECT2.csv and DOMEENVI.csv
+		# byte offset
+		try:
+			e[14] = int(e[14])
+		except ValueError:	# we can ignore invalid entries
+			return []
+		# ID
+		if len(e[12]) == 0:	# we can ignore entries with an empty ID
+			return []
+		# fill ID with zeros
+		e[12] = e[12].zfill(7)
+		# skipped numbers: "ketsu-ban" in Shift-JIS
+		if e[1] == '\x8c\x87\x94\xd4':
+			return []
+		# for future use
+		elif e[12] == '0006049':
+			return []
+		# some names have a trailing space
+		e[1] = e[1].strip()
+		# some decimal positions have a trailing space
+		e[15] = e[15].strip()
+		# number of bits
+		e[2] = int(e[2])
+		# decription
+		# e[5] = some way to convert encoding
+		# format
+		e[9] = string.upper(e[9])
+
+		# ID specfic typos and expansions
+		idnum = string.atoi(e[12],16)
+		# id:002B07F seems not to be a BCD but a bit field 
+		if e[12] == '002B07F':
+			e[9] = ''
+		# id:000E02C in CALMON.csv has a typo
+		elif e[12] == '000E02C':
+			e[1] = 'Cal Source TurretD Holder3 Selected'
+		# id:002A0FC in DOMEENVI.csv has a typo
+		elif e[12] == '002A0FC':
+			e[1] = 'Ns FL Rear V WVEL Data'
+		# id:002A174 in DOMEENVI.csv has a typo
+		elif e[12] == '002A174':
+			e[1] = 'Top FL Rear V WVEL Data'
+		# id:002C295-002C2A0 in TLSCPENV.csv have typos
+		elif 0x002C295 <= idnum and idnum < 0x002C2A1:
+			e[1] = re.sub('IR-B', 'Opt-B', e[1])
+		# id:002D100 in TLSCPCT2.csv has a typo
+		elif e[12] == '002D100':
+			e[1] = 'CSCT(F-IR) CT2 OVER RNG'
+		# id:00B218B has to be expanded to many of Zernike polynominals
+		# source: MLP2-MLP2_1.h by Morino
+		elif e[12] == '00B218B':
+			r = []
+			offset = int(e[14])
+			template = e[:]
+			template[2] = 64
+			template[9] = 'REAL'
+			for i in range(0,len(TelStatusCsv.zernike_terms)):
+				if len(TelStatusCsv.zernike_terms[i]) > 0:
+					template[1] = "%s %02d A%s" % (e[1],i,TelStatusCsv.zernike_terms[i])
+				else:
+					template[1] = "%s %02d" % (e[1],i)
+				template[14] = offset
+				r.append(template[:])
+				offset += template[2]/8
+			return r
+		# id:0040001 has to be expanded to many of Zernike polynominals
+		# source: SH_TEST-SHTESTMON.h by Morino
+		elif e[12] == '0040001':
+			r = []
+			offset = int(e[14])
+			template = e[:]
+			template[2] = 16
+			template[9] = 'BINARY'
+			template[15] = '0.01'
+			for i in range(1,66):
+				template[1] = "%s %02d A%s" % (e[1],i,TelStatusCsv.zernike_terms[i])
+				template[14] = offset
+				r.append(template[:])
+				offset += template[2]/8
+			return r
+		# id:0040004 has to be expanded to many polynominals
+		# source: SH_TEST-SHTESTMON.h by Morino
+		elif e[12] == '0040004':
+			r = []
+			offset = int(e[14])
+			template = e[:]
+			template[2] = 64
+			template[9] = 'REAL'
+			for i in range(1,33):
+				template[1] = "%s q%02d" % (e[1],i)
+				template[14] = offset
+				r.append(template[:])
+				offset += template[2]/8
+			return r
+		# id:0040015 has to be expanded to many polynominals
+		# source: SH_TEST-SHTESTMON.h by Morino
+		elif e[12] == '0040015':
+			r = []
+			offset = int(e[14])
+			template = e[:]
+			template[2] = 16
+			template[9] = 'BINARY'
+			template[15] = '0.01'
+			for i in range(0,192):
+				template[1] = "%s %03d" % (e[1],i)
+				template[14] = offset
+				r.append(template[:])
+				offset += template[2]/8
+			return r
+		# entries in PMAMON.csv has to be expanded to those of #000-260
+		# also for name_and_total
+		# http://www1.naoj.org/telgroup/TSC_Status_html/sig/Pmamon.htm
+		# http://www1.naoj.org/telgroup/TSC_Command.html/Individual/TWS/TWS_2_3.htm
+		elif e[12][0:4] == '0101':
+			r = []
+			template = e[:]
+			for n1 in range(1,9):
+				for n2 in range(0,7):
+					for n3 in range(0,10):
+						numstr = "%d%d%d" % (n1, n2, n3)
+						template[1] = "%s%d-%d%d%s" % (e[1][0:8], n1, n2, n3, e[1][12:])
+						template[12] = e[12][0] + numstr + e[12][4:7]
+						r.append(template[:])
+						mitsubishi.name_and_total[template[12][0:4]] = {'name':'PMA-'+numstr}
+			return r
+		return [e]
+
 	# parse
 	def parse(self):
 		r = []
 		for col in csv.reader(file(self.path)):
-			x = self.corrected(col)
 			for c in self.corrected(col):
-				e = TelStatusEntry(c[1], c[2], c[9], c[12], c[13], c[14], c[15], self.basename, {'ja':c[5]})
+				if c[6] == "1":
+					isalarm = True
+				else:
+					isalarm = False
+				if c[7] == "1":
+					iswarning = True
+				else:
+					iswarning = False
+				if c[8] == "1":
+					isfault = True
+				else:
+					isfault = False
+				e = TelStatusEntry(c[1], c[2], isalarm, iswarning, isfault, c[9], c[12], c[13], c[14], c[15], self.basename, {'ja':c[5]})
 				r.append(e)
 		return r
-
