@@ -1,4 +1,4 @@
-/* ANSI-C code produced by gperf version 3.0.1 */
+/* ANSI-C code produced by gperf version 3.0.4 */
 /* Command-line: /usr/bin/gperf -I -t -L ANSI-C -N melco_tree_lookup -T tree-hash.txt  */
 /* Computed positions: -k'2-4,$' */
 
@@ -1380,6 +1380,9 @@ hash (register const char *str, register unsigned int len)
 
 #ifdef __GNUC__
 __inline
+#if defined __GNUC_STDC_INLINE__ || defined __GNUC_GNU_INLINE__
+__attribute__ ((__gnu_inline__))
+#endif
 #endif
 struct melco_tree_s *
 melco_tree_lookup (register const char *str, register unsigned int len)
